# Manage Unix hosts.  We largely just modify /etc/hosts, except in the case
# of OS X where we have to use NetInfo.

require 'etc'
require 'facter'
require 'puppet/type/state'
require 'puppet/type/nameservice'

module Puppet
    class State
        module HostIP
            def self.doc
                "The host's IP address."
            end

            def self.name
                :ip
            end

            def shouldprocess(ip)
                return ip if ip.is_a? Symbol

                unless ip =~ /{\d+\.}[3]\d+./
                    raise Puppet::Error, "Invalid IP address %s" % ip
                end
                ip.split(".").each { |oct|
                    if oct.to_i > 255 or oct.to_ < 0
                        raise Puppet::Error, "Invalid IP address %s" % ip
                    end
                }

                self.info "Setting IP address to %s" % ip

                return ip
            end
        end

        module HostAlias
            def self.doc
                "Any aliases for the IP address."
            end

            def self.name
                :alias
            end

            def shouldprocess(name)
                return name if ip.is_a? Symbol

                unless name.is_a? String
                    raise Puppet::Error, "Invalid alias %s" % name
                end

                self.info "Adding alias %s" % name

                return name
            end
        end
    end

    class Type
        class Host < Type
            statenames = [
                "HostIP", "HostAlias"
            ]
            case Facter["operatingsystem"].value
            when "Darwin":
                @statemodule = Puppet::NameService::NetInfo
            else
                @statemodule = Puppet::NameService::ObjectAdd
            end

            @states = statenames.collect { |name|
                fullname = @statemodule.to_s + "::" + name
                begin
                    eval(fullname)
                rescue NameError
                    raise Puppet::DevError, "Could not retrieve state class %s" %
                        fullname
                end
            }.each { |klass|
                klass.complete
            }

            @name = :group
            @namevar = :name

            @parameters = [:name]

            class << self
                attr_accessor :netinfodir
                attr_accessor :statemodule
            end

            @netinfodir = "groups"

            @paramdoc[:name] = "The group name.  While naming limitations vary by
                system, it is advisable to keep the name to the degenerate
                limitations, which is a maximum of 8 characters beginning with
                a letter."

            @doc = "Manage groups.  This type can only create groups.  Group
                membership must be managed on individual users."

            def exists?
                self.class.statemodule.exists?(self)
            end

            def getinfo(refresh = false)
                if @groupinfo.nil? or refresh == true
                    begin
                        @groupinfo = Etc.getgrnam(self.name)
                    rescue ArgumentError => detail
                        @groupinfo = nil
                    end
                end

                @groupinfo
            end

            def initialize(hash)
                @groupinfo = nil
                super
            end

            def retrieve
                if self.exists?
                    super
                else
                    # the group does not exist
                    unless @states.include?(:gid)
                        self[:gid] = :auto
                    end

                    @states.each { |name, state|
                        state.is = :notfound
                    }

                    return
                end
            end
        end
    end
end

# $Id: group.rb 731 2005-10-26 04:44:25Z luke $
